// Simple serial port capture program
// Port and baud rate are hard coded
// Ctrl-c handler should avoid loss of trace data. But eventually
//   a part is missing which was not received so far. Fix trace file content then manually.
//
// To generate graphics call: mscgen -T png -i filename -o filename.png

#include <stdio.h>   
#include <stdlib.h>   
#include <string.h>  
#include <unistd.h>  
#include <fcntl.h>   
#include <errno.h>   
#include <termios.h> 
#include <signal.h>
#include <time.h>

// states of the program
#define INIT 0
#define RUN 1


FILE *fp;
char filename[255]={0};

int open_port(void);
FILE* open_file(char*);

void intHandler(int signal) {
  	fprintf(fp,"\n\n}\n");// reinit
	  printf("\n\n}\n");
    fflush(fp);
    fclose(fp);
    exit(0);
}

int main(int argc, char* argv[]){

	int statevar=INIT;
	char buf[255]={};
  struct sigaction act;
  
  // register signal handler to catch ctrl c
  act.sa_handler = intHandler;
  sigaction(SIGINT, &act, NULL);

	if(argc!=2){
		perror("usage: capture_mscflow filename \n");
		exit(1);
	}

	fp = open_file(argv[1]);
	int fd = open_port();
	
	while(1){
		int n;
		memset(buf,0,255);
		switch(statevar){
	
			case INIT:
				n=read(fd,buf,9);
				if(strncmp("__start__",buf,9)==0){
					statevar=RUN;
				}
			break;
			
			case RUN:
				n=read(fd,buf,255);
				if(n!=0){
					if(strncmp("__start__",buf,9)==0){
						fprintf(fp,"\n\n}\n");// reinit
						printf("\n\n}\n");
						
						// close old file
						fclose(fp);
						// open new file
						fp=open_file(argv[1]);
  					fprintf(fp,"%s",buf+9); // print remaining content of buffer
						printf("%s",buf+9); // print remaining content of buffer
					}else{
						buf[n+1]=0;
						fprintf(fp,"%s",buf);
						printf("%s",buf);
					}
				}
			break;
		}
	}

	return 0;

}

FILE* open_file(char* name){
	static int i=0;// increasing file postfix
	time_t t;
	struct tm * ts;

	t = time(NULL);
	ts = localtime(&t);

	sprintf(filename,"%s_%d%d%d%d%d_%d.dat", name, ts->tm_year+1900, ts->tm_mon+1, ts->tm_mday, ts->tm_hour, ts->tm_min, ts->tm_sec);
	FILE* fp = fopen(filename, "we");
	if(!fp){
		perror("Error opening file - \n");
		exit(1);
	}
	return fp;
}

int open_port(void)
{
  int fd; /* File descriptor for the port */
  struct termios options;

  fd = open("/dev/cu.usbmodem413", O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd == -1)
  {
   /*
    * Could not open the port.
    */

    perror("open_port: Unable to open serial port - ");
    exit(1);
  }
  else{
    fcntl(fd, F_SETFL, 0); // blocking
	/*
	 * Get the current options for the port...
	 */

	tcgetattr(fd, &options);

	/*
	 * Set the baud rates to 19200...
	 */

	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);

	/*
	 * Enable the receiver and set local mode...
	 */

	options.c_cflag |= (CLOCAL | CREAD);

	/*
	 * Set the new options for the port...
	 */

	tcsetattr(fd, TCSANOW, &options);
	}

  return (fd);
}
